pub const fn assert_incompatible_size_align<Max, Min>()Expand description
Assert that size and alignment of type Max is greater than size and alignment of type Min at compile time.
Usage
#[repr(C)]
struct Fred {
foo: u32,
bar: u64,
baz: bool,
}
const _: () = isit::assert_incompatible_size_align::<Fred, u32>();